@@ -1,6 +1,6 @@
-# Copyright 1999-2006 Gentoo Foundation
+# Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: isolated-functions.sh 12135 2008-12-02 22:20:34Z zmedico $
+# $Id: isolated-functions.sh 14527 2009-10-09 19:50:09Z zmedico $
 
 # We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
@@ -53,7 +53,24 @@ dump_trace() {
 	done
 }
 
+nonfatal() {
+	if has "${EAPI:-0}" 0 1 2; then
+		die "$FUNCNAME() not supported in this EAPI"
+	fi
+	if [[ $# -lt 1 ]]; then
+		die "$FUNCNAME(): Missing argument"
+	fi
+
+	PORTAGE_NONFATAL=1 "$@"
+}
+
 die() {
+	if [[ $PORTAGE_NONFATAL -eq 1 ]]; then
+		echo -e " $WARN*$NORMAL ${FUNCNAME[1]}: WARNING: $@" >&2
+		return 1
+	fi
+
+	set +e
 	if [ -n "${QA_INTERCEPTORS}" ] ; then
 		# die was called from inside inherit. We need to clean up
 		# QA_INTERCEPTORS since sed is called below.
@@ -62,15 +79,23 @@ die() {
 	fi
 	local n filespacing=0 linespacing=0
 	# setup spacing to make output easier to read
-	for ((n = ${#FUNCNAME[@]} - 1; n >= 0; --n)); do
+	(( n = ${#FUNCNAME[@]} - 1 ))
+	while (( n > 0 )) ; do
+		[ "${FUNCNAME[${n}]}" == "qa_call" ] && break
+		(( n-- ))
+	done
+	(( n == 0 )) && (( n = ${#FUNCNAME[@]} - 1 ))
+	while (( n > 0 )); do
 		sourcefile=${BASH_SOURCE[${n}]} sourcefile=${sourcefile##*/}
 		lineno=${BASH_LINENO[${n}]}
 		((filespacing < ${#sourcefile})) && filespacing=${#sourcefile}
 		((linespacing < ${#lineno}))     && linespacing=${#lineno}
+		(( n-- ))
 	done
 
+	eerror "ERROR: $CATEGORY/$PF failed:"
+	eerror "  ${*:-(no error message)}"
 	eerror
-	eerror "ERROR: $CATEGORY/$PF failed."
 	dump_trace 2 ${filespacing} ${linespacing}
 	eerror "  $(printf "%${filespacing}s" "${BASH_SOURCE[1]##*/}"), line $(printf "%${linespacing}s" "${BASH_LINENO[0]}"):  Called die"
 	eerror "The specific snippet of code:"
@@ -95,21 +120,9 @@ die() {
 		${BASH_SOURCE[1]} \
 		| sed -e '1d' -e 's:^:RETAIN-LEADING-SPACE:' \
 		| while read -r n ; do eerror "  ${n#RETAIN-LEADING-SPACE}" ; done
-	eerror " The die message:"
-	eerror "  ${*:-(no error message)}"
 	eerror
-	eerror "If you need support, post the topmost build error, and the call stack if relevant."
-	[[ -n ${PORTAGE_LOG_FILE} ]] \
-		&& eerror "A complete build log is located at '${PORTAGE_LOG_FILE}'."
-	if [ -f "${T}/environment" ] ; then
-		eerror "The ebuild environment file is located at '${T}/environment'."
-	elif [ -d "${T}" ] ; then
-		{
-			set
-			export
-		} > "${T}/die.env"
-		eerror "The ebuild environment file is located at '${T}/die.env'."
-	fi
+	eerror "If you need support, post the output of 'emerge --info =$CATEGORY/$PF',"
+	eerror "the complete build log and the output of 'emerge -pqv =$CATEGORY/$PF'."
 	if [[ -n ${EBUILD_OVERLAY_ECLASSES} ]] ; then
 		eerror "This ebuild used the following eclasses from overlays:"
 		local x
@@ -123,9 +136,20 @@ die() {
 		local overlay=${EBUILD%/*}
 		overlay=${overlay%/*}
 		overlay=${overlay%/*}
-		eerror "This ebuild is from an overlay: '${overlay}/'"
+		if [[ -n $PORTAGE_REPO_NAME ]] ; then
+			eerror "This ebuild is from an overlay named" \
+				"'$PORTAGE_REPO_NAME': '${overlay}/'"
+		else
+			eerror "This ebuild is from an overlay: '${overlay}/'"
+		fi
+	elif [[ -n $PORTAGE_REPO_NAME && -f "$PORTDIR"/profiles/repo_name ]] ; then
+		local portdir_repo_name=$(<"$PORTDIR"/profiles/repo_name)
+		if [[ -n $portdir_repo_name && \
+			$portdir_repo_name != $PORTAGE_REPO_NAME ]] ; then
+			eerror "This ebuild is from a repository" \
+				"named '$PORTAGE_REPO_NAME'"
+		fi
 	fi
-	eerror
 
 	if [[ "${EBUILD_PHASE/depend}" == "${EBUILD_PHASE}" ]] ; then
 		local x
@@ -134,10 +158,23 @@ die() {
 		done
 	fi
 
+	[[ -n ${PORTAGE_LOG_FILE} ]] \
+		&& eerror "The complete build log is located at '${PORTAGE_LOG_FILE}'."
+	if [ -f "${T}/environment" ] ; then
+		eerror "The ebuild environment file is located at '${T}/environment'."
+	elif [ -d "${T}" ] ; then
+		{
+			set
+			export
+		} > "${T}/die.env"
+		eerror "The ebuild environment file is located at '${T}/die.env'."
+	fi
+	eerror "S: '${S}'"
+
 	[ -n "$EBUILD_EXIT_STATUS_FILE" ] && > "$EBUILD_EXIT_STATUS_FILE"
 
 	# subshell die support
-	kill -s SIGTERM ${EBUILD_MASTER_PID}
+	[[ $BASHPID = $EBUILD_MASTER_PID ]] || kill -s SIGTERM $EBUILD_MASTER_PID
 	exit 1
 }
 
@@ -169,7 +206,7 @@ elog_base() {
 			return 1
 			;;
 	esac
-	echo -e "$@" | while read ; do
+	echo -e "$@" | while read -r ; do
 		echo "$messagetype $REPLY" >> \
 			"${T}/logging/${EBUILD_PHASE:-other}"
 	done
@@ -179,8 +216,8 @@ elog_base() {
 eqawarn() {
 	elog_base QA "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e "$@" | while read ; do
-		vecho -e " $WARN*$NORMAL $REPLY" >&2
+	echo -e "$@" | while read -r ; do
+		vecho " $WARN*$NORMAL $REPLY" >&2
 	done
 	LAST_E_CMD="eqawarn"
 	return 0
@@ -189,8 +226,8 @@ eqawarn() {
 elog() {
 	elog_base LOG "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e "$@" | while read ; do
-		echo -e " $GOOD*$NORMAL $REPLY"
+	echo -e "$@" | while read -r ; do
+		echo " $GOOD*$NORMAL $REPLY"
 	done
 	LAST_E_CMD="elog"
 	return 0
@@ -217,8 +254,8 @@ esyslog() {
 einfo() {
 	elog_base INFO "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e "$@" | while read ; do
-		echo -e " $GOOD*$NORMAL $REPLY"
+	echo -e "$@" | while read -r ; do
+		echo " $GOOD*$NORMAL $REPLY"
 	done
 	LAST_E_CMD="einfo"
 	return 0
@@ -235,8 +272,8 @@ einfon() {
 ewarn() {
 	elog_base WARN "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e "$@" | while read ; do
-		echo -e " $WARN*$NORMAL $RC_INDENTATION$REPLY" >&2
+	echo -e "$@" | while read -r ; do
+		echo " $WARN*$NORMAL $RC_INDENTATION$REPLY" >&2
 	done
 	LAST_E_CMD="ewarn"
 	return 0
@@ -245,8 +282,8 @@ ewarn() {
 eerror() {
 	elog_base ERROR "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
-	echo -e "$@" | while read ; do
-		echo -e " $BAD*$NORMAL $RC_INDENTATION$REPLY" >&2
+	echo -e "$@" | while read -r ; do
+		echo " $BAD*$NORMAL $RC_INDENTATION$REPLY" >&2
 	done
 	LAST_E_CMD="eerror"
 	return 0
@@ -488,7 +525,8 @@ save_ebuild_env() {
 		unset -f dump_trace die diefunc quiet_mode vecho elog_base eqawarn elog \
 			esyslog einfo einfon ewarn eerror ebegin _eend eend KV_major \
 			KV_minor KV_micro KV_to_int get_KV unset_colors set_colors has \
-			hasv hasq qa_source qa_call addread addwrite adddeny addpredict \
+			hasg hasgq hasv hasq qa_source qa_call \
+			addread addwrite adddeny addpredict _sb_append_var \
 			lchown lchgrp esyslog use usev useq has_version portageq \
 			best_version use_with use_enable register_die_hook check_KV \
 			keepdir unpack strip_duplicate_slashes econf einstall \
@@ -504,23 +542,24 @@ save_ebuild_env() {
 			source_all_bashrcs ebuild_main \
 			ebuild_phase ebuild_phase_with_hooks \
 			_ebuild_arg_to_phase _ebuild_phase_funcs default \
+			_source_ebuild \
 			${QA_INTERCEPTORS}
 
 		# portage config variables and variables set directly by portage
-		unset BAD BRACKET BUILD_PREFIX COLS \
+		unset ACCEPT_LICENSE BAD BRACKET BUILD_PREFIX COLS \
 			DISTCC_DIR DISTDIR DOC_SYMLINKS_DIR \
 			EBUILD_EXIT_STATUS_FILE EBUILD_FORCE_TEST EBUILD_MASTER_PID \
 			ECLASSDIR ECLASS_DEPTH ENDCOL FAKEROOTKEY \
 			GOOD HILITE HOME IMAGE \
 			LAST_E_CMD LAST_E_LEN LD_PRELOAD MISC_FUNCTIONS_ARGS MOPREFIX \
-			NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
+			NOCOLOR NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
 			PORTAGE_ACTUAL_DISTDIR PORTAGE_ARCHLIST \
 			PORTAGE_BASHRC PORTAGE_BASHRCS_SOURCED \
 			PORTAGE_BINPKG_TAR_OPTS PORTAGE_BINPKG_TMPFILE PORTAGE_BUILDDIR \
 			PORTAGE_COLORMAP PORTAGE_CONFIGROOT PORTAGE_DEBUG \
 			PORTAGE_DEPCACHEDIR PORTAGE_GID PORTAGE_INST_GID \
 			PORTAGE_INST_UID PORTAGE_LOG_FILE PORTAGE_MASTER_PID \
-			PORTAGE_QUIET \
+			PORTAGE_NONFATAL PORTAGE_QUIET \
 			PORTAGE_REPO_NAME PORTAGE_RESTRICT PORTAGE_UPDATE_ENV \
 			PORTAGE_VERBOSE PORTAGE_WORKDIR_MODE PORTDIR \
 			PORTDIR_OVERLAY ${!PORTAGE_SANDBOX_*} PREROOTPATH \
