@@ -1,7 +1,7 @@
 #!/bin/bash
-# Copyright 1999-2006 Gentoo Foundation
+# Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header$
+# $Id: misc-functions.sh 14531 2009-10-09 22:13:04Z zmedico $
 #
 # Miscellaneous shell functions that make use of the ebuild env but don't need
 # to be included directly in ebuild.sh.
@@ -44,6 +44,7 @@ install_symlink_html_docs() {
 install_qa_check() {
 	cd "${D}" || die "cd failed"
 
+	export STRIP_MASK
 	prepall
 	ecompressdir --dequeue
 	ecompress --dequeue
@@ -67,19 +68,24 @@ install_qa_check() {
 			unset PORTAGE_QUIET
 		fi
 
-		# Make sure we disallow insecure RUNPATH/RPATH's
+		# Make sure we disallow insecure RUNPATH/RPATHs
 		# Don't want paths that point to the tree where the package was built
 		# (older, broken libtools would do this).  Also check for null paths
 		# because the loader will search $PWD when it finds null paths.
 		f=$(scanelf -qyRF '%r %p' "${D}" | grep -E "(${PORTAGE_BUILDDIR}|: |::|^:|^ )")
-		if [[ -n ${f} ]] ; then
+		# Reject set*id binaries with $ORIGIN in RPATH #260331
+		x=$(
+			find "${D}" -type f \( -perm -u+s -o -perm -g+s \) -print0 | \
+			xargs -0 scanelf -qyRF '%r %p' | grep '$ORIGIN'
+		)
+		if [[ -n ${f}${x} ]] ; then
 			vecho -ne '\a\n'
-			eqawarn "QA Notice: The following files contain insecure RUNPATH's"
+			eqawarn "QA Notice: The following files contain insecure RUNPATHs"
 			eqawarn " Please file a bug about this at http://bugs.gentoo.org/"
 			eqawarn " with the maintaining herd of the package."
-			eqawarn "${f}"
+			eqawarn "${f}${f:+${x:+\n}}${x}"
 			vecho -ne '\a\n'
-			if has stricter ${FEATURES} ; then
+			if [[ -n ${x} ]] || has stricter ${FEATURES} ; then
 				insecure_rpath=1
 			else
 				vecho "Auto fixing rpaths for ${f}"
@@ -87,7 +93,7 @@ install_qa_check() {
 			fi
 		fi
 
-		# TEXTREL's are baaaaaaaad
+		# TEXTRELs are baaaaaaaad
 		# Allow devs to mark things as ignorable ... e.g. things that are
 		# binary-only and upstream isn't cooperating (nvidia-glx) ... we
 		# allow ebuild authors to set QA_TEXTRELS_arch and QA_TEXTRELS ...
@@ -98,7 +104,7 @@ install_qa_check() {
 		export QA_TEXTRELS="${QA_TEXTRELS} lib*/modules/*.ko"
 		f=$(scanelf -qyRF '%t %p' "${D}" | grep -v 'usr/lib/debug/')
 		if [[ -n ${f} ]] ; then
-			scanelf -qyRF '%T %p' "${PORTAGE_BUILDDIR}"/ &> "${T}"/scanelf-textrel.log
+			scanelf -qyRAF '%T %p' "${PORTAGE_BUILDDIR}"/ &> "${T}"/scanelf-textrel.log
 			vecho -ne '\a\n'
 			eqawarn "QA Notice: The following files contain runtime text relocations"
 			eqawarn " Text relocations force the dynamic linker to perform extra"
@@ -136,21 +142,23 @@ install_qa_check() {
 					[[ -n ${QA_STRICT_WX_LOAD} ]] && QA_WX_LOAD=""
 					export QA_EXECSTACK="${QA_EXECSTACK} lib*/modules/*.ko"
 					export QA_WX_LOAD="${QA_WX_LOAD} lib*/modules/*.ko"
-					f=$(scanelf -qyRF '%e %p' "${D}" | grep -v 'usr/lib/debug/')
+					f=$(scanelf -qyRAF '%e %p' "${D}" | grep -v 'usr/lib/debug/')
 					;;
 			esac
 			;;
 		esac
 		if [[ -n ${f} ]] ; then
 			# One more pass to help devs track down the source
-			scanelf -qyRF '%e %p' "${PORTAGE_BUILDDIR}"/ &> "${T}"/scanelf-execstack.log
+			scanelf -qyRAF '%e %p' "${PORTAGE_BUILDDIR}"/ &> "${T}"/scanelf-execstack.log
 			vecho -ne '\a\n'
-			eqawarn "QA Notice: The following files contain executable stacks"
-			eqawarn " Files with executable stacks will not work properly (or at all!)"
-			eqawarn " on some architectures/operating systems.  A bug should be filed"
-			eqawarn " at http://bugs.gentoo.org/ to make sure the file is fixed."
+			eqawarn "QA Notice: The following files contain writable and executable sections"
+			eqawarn " Files with such sections will not work properly (or at all!) on some"
+			eqawarn " architectures/operating systems.  A bug should be filed at"
+			eqawarn " http://bugs.gentoo.org/ to make sure the issue is fixed."
 			eqawarn " For more information, see http://hardened.gentoo.org/gnu-stack.xml"
 			eqawarn " Please include the following list of files in your report:"
+			eqawarn " Note: Bugs should be filed for the respective maintainers"
+			eqawarn " of the package in question and not hardened@g.o."
 			eqawarn "${f}"
 			vecho -ne '\a\n'
 			die_msg="${die_msg} execstacks"
@@ -159,6 +167,8 @@ install_qa_check() {
 
 		# Check for files built without respecting LDFLAGS
 		if [[ "${LDFLAGS}" == *--hash-style=gnu* ]] && [[ "${PN}" != *-bin ]] ; then
+			qa_var="QA_DT_HASH_${ARCH/-/_}"
+			eval "[[ -n \${!qa_var} ]] && QA_DT_HASH=(\"\${${qa_var}[@]}\")"
 			f=$(scanelf -qyRF '%k %p' -k .hash "${D}" | sed -e "s:\.hash ::")
 			if [[ -n ${f} ]] ; then
 				echo "${f}" > "${T}"/scanelf-ignored-LDFLAGS.log
@@ -233,26 +243,73 @@ install_qa_check() {
 			die "Aborting due to QA concerns: ${die_msg}"
 		fi
 
-		# Run some sanity checks on shared libraries
-		for d in "${D}"lib* "${D}"usr/lib* ; do
-			f=$(scanelf -ByF '%S %p' "${d}"/lib*.so* | gawk '$2 == "" { print }')
+		# Check for shared libraries lacking SONAMEs
+		qa_var="QA_SONAME_${ARCH/-/_}"
+		eval "[[ -n \${!qa_var} ]] && QA_SONAME=(\"\${${qa_var}[@]}\")"
+		f=$(scanelf -ByF '%S %p' "${D}"{,usr/}lib*/lib*.so* | gawk '$2 == "" { print }' | sed -e "s:^[[:space:]]${D}:/:")
+		if [[ -n ${f} ]] ; then
+			echo "${f}" > "${T}"/scanelf-missing-SONAME.log
+			if [[ "${QA_STRICT_SONAME-unset}" == unset ]] ; then
+				if [[ ${#QA_SONAME[@]} -gt 1 ]] ; then
+					for x in "${QA_SONAME[@]}" ; do
+						sed -e "s#^/${x#/}\$##" -i "${T}"/scanelf-missing-SONAME.log
+					done
+				else
+					local shopts=$-
+					set -o noglob
+					for x in ${QA_SONAME} ; do
+						sed -e "s#^/${x#/}\$##" -i "${T}"/scanelf-missing-SONAME.log
+					done
+					set +o noglob
+					set -${shopts}
+				fi
+			fi
+			sed -e "/^\$/d" -i "${T}"/scanelf-missing-SONAME.log
+			f=$(<"${T}"/scanelf-missing-SONAME.log)
 			if [[ -n ${f} ]] ; then
 				vecho -ne '\a\n'
 				eqawarn "QA Notice: The following shared libraries lack a SONAME"
 				eqawarn "${f}"
 				vecho -ne '\a\n'
 				sleep 1
+			else
+				rm -f "${T}"/scanelf-missing-SONAME.log
 			fi
+		fi
 
-			f=$(scanelf -ByF '%n %p' "${d}"/lib*.so* | gawk '$2 == "" { print }')
+		# Check for shared libraries lacking NEEDED entries
+		qa_var="QA_DT_NEEDED_${ARCH/-/_}"
+		eval "[[ -n \${!qa_var} ]] && QA_DT_NEEDED=(\"\${${qa_var}[@]}\")"
+		f=$(scanelf -ByF '%n %p' "${D}"{,usr/}lib*.so* | gawk '$2 == "" { print }' | sed -e "s:^[[:space:]]${D}:/:")
+		if [[ -n ${f} ]] ; then
+			echo "${f}" > "${T}"/scanelf-missing-NEEDED.log
+			if [[ "${QA_STRICT_DT_NEEDED-unset}" == unset ]] ; then
+				if [[ ${#QA_DT_NEEDED[@]} -gt 1 ]] ; then
+					for x in "${QA_DT_NEEDED[@]}" ; do
+						sed -e "s#^/${x#/}\$##" -i "${T}"/scanelf-missing-NEEDED.log
+					done
+				else
+					local shopts=$-
+					set -o noglob
+					for x in ${QA_DT_NEEDED} ; do
+						sed -e "s#^/${x#/}\$##" -i "${T}"/scanelf-missing-NEEDED.log
+					done
+					set +o noglob
+					set -${shopts}
+				fi
+			fi
+			sed -e "/^\$/d" -i "${T}"/scanelf-missing-NEEDED.log
+			f=$(<"${T}"/scanelf-missing-NEEDED.log)
 			if [[ -n ${f} ]] ; then
 				vecho -ne '\a\n'
 				eqawarn "QA Notice: The following shared libraries lack NEEDED entries"
 				eqawarn "${f}"
 				vecho -ne '\a\n'
 				sleep 1
+			else
+				rm -f "${T}"/scanelf-missing-NEEDED.log
 			fi
-		done
+		fi
 
 		PORTAGE_QUIET=${tmp_quiet}
 	fi
@@ -351,8 +408,16 @@ install_qa_check() {
 
 	# Evaluate misc gcc warnings
 	if [[ -n ${PORTAGE_LOG_FILE} && -r ${PORTAGE_LOG_FILE} ]] ; then
+		# In debug mode, this variable definition and corresponding grep calls
+		# will produce false positives if they're shown in the trace.
+		local reset_debug=0
+		if [[ ${-/x/} != $- ]] ; then
+			set +x
+			reset_debug=1
+		fi
 		local m msgs=(
 			": warning: dereferencing type-punned pointer will break strict-aliasing rules$"
+			": warning: dereferencing pointer .* does break strict-aliasing rules$"
 			": warning: implicit declaration of function "
 			": warning: incompatible implicit declaration of built-in function "
 			": warning: is used uninitialized in this function$" # we'll ignore "may" and "might"
@@ -374,7 +439,9 @@ install_qa_check() {
 				abort="yes"
 			fi
 		done
-		f=$(cat "${PORTAGE_LOG_FILE}" | check-implicit-pointer-usage.py)
+		[[ $reset_debug = 1 ]] && set -x
+		f=$(cat "${PORTAGE_LOG_FILE}" | \
+			"$PORTAGE_BIN_PATH"/check-implicit-pointer-usage.py)
 		if [[ -n ${f} ]] ; then
 
 			# In the future this will be a forced "die". In preparation,
@@ -383,7 +450,7 @@ install_qa_check() {
 
 			# just warn on 32bit hosts but bail on 64bit hosts
 			case ${CHOST} in
-				alpha*|ia64*|powerpc64*|mips64*|sparc64*|x86_64*) gentoo_bug=yes ;;
+				alpha*|hppa64*|ia64*|powerpc64*|mips64*|sparc64*|sparcv9*|x86_64*) gentoo_bug=yes ;;
 			esac
 
 			abort=yes
@@ -407,14 +474,20 @@ install_qa_check() {
 			fi
 
 		fi
-		if [[ $abort = yes ]] && [[ $gentoo_bug != yes ]] ; then
-			echo "Please do not file a Gentoo bug and instead" \
-			"report the above QA issues directly to the upstream" \
-			"developers of this software." | fmt -w 70 | \
-			while read line ; do eqawarn "${line}" ; done
-			eqawarn "Homepage: ${HOMEPAGE}"
+		if [[ ${abort} == "yes" ]] ; then
+			if [[ ${gentoo_bug} == "yes" ]] ; then
+				die "install aborted due to" \
+					"poor programming practices shown above"
+			else
+				echo "Please do not file a Gentoo bug and instead" \
+				"report the above QA issues directly to the upstream" \
+				"developers of this software." | fmt -w 70 | \
+				while read line ; do eqawarn "${line}" ; done
+				eqawarn "Homepage: ${HOMEPAGE}"
+				hasq stricter ${FEATURES} && die "install aborted due to" \
+					"poor programming practices shown above"
+			fi
 		fi
-		[[ ${abort} == "yes" ]] && hasq stricter ${FEATURES} && die "poor code kills airplanes"
 	fi
 
 	# Compiled python objects do not belong in /usr/share (FHS violation)
@@ -479,8 +552,7 @@ install_mask() {
 }
 
 preinst_bsdflags() {
-	type -P chflags > /dev/null || return 0
-	type -P mtree > /dev/null || return 1
+	hasq chflags $FEATURES || return
 	# Save all the file flags for restoration after installation.
 	mtree -c -p "${D}" -k flags > "${T}/bsdflags.mtree"
 	# Remove all the file flags so that the merge phase can do anything
@@ -490,8 +562,7 @@ preinst_bsdflags() {
 }
 
 postinst_bsdflags() {
-	type -P chflags > /dev/null || return 0
-	type -P mtree > /dev/null || return 1
+	hasq chflags $FEATURES || return
 	# Restore all the file flags that were saved before installation.
 	mtree -e -p "${ROOT}" -U -k flags < "${T}/bsdflags.mtree" &> /dev/null
 }
@@ -637,8 +708,8 @@ dyn_package() {
 		PORTAGE_BINPKG_TMPFILE="${PKGDIR}/${CATEGORY}/${PF}.tbz2"
 	mkdir -p "${PORTAGE_BINPKG_TMPFILE%/*}" || die "mkdir failed"
 	tar $tar_options -cf - $PORTAGE_BINPKG_TAR_OPTS -C "${D}" . | \
-		bzip2 -f > "$PORTAGE_BINPKG_TMPFILE"
-	assert
+		bzip2 -cf > "$PORTAGE_BINPKG_TMPFILE"
+	assert "failed to pack binary package: '$PORTAGE_BINPKG_TMPFILE'"
 	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	python -c "from portage import xpak; t=xpak.tbz2('${PORTAGE_BINPKG_TMPFILE}'); t.recompose('${PORTAGE_BUILDDIR}/build-info')"
 	if [ $? -ne 0 ]; then
