@@ -1,13 +1,22 @@
 # Copyright: 2005 Gentoo Foundation
 # Author(s): Brian Harring (ferringb@gentoo.org)
 # License: GPL2
-# $Id: flat_hash.py 11985 2008-11-17 23:10:27Z zmedico $
+# $Id: flat_hash.py 14416 2009-09-24 22:15:48Z zmedico $
 
+import codecs
 from portage.cache import fs_template
 from portage.cache import cache_errors
-import errno, os, stat
-from portage.cache.template import reconstruct_eclasses
-# store the current key order *here*.
+import errno
+import stat
+import sys
+import os as _os
+from portage import os
+from portage import _encodings
+from portage import _unicode_encode
+
+if sys.hexversion >= 0x3000000:
+	long = int
+
 class database(fs_template.FsBased):
 
 	autocommits = True
@@ -23,55 +32,56 @@ class database(fs_template.FsBased):
 		if not self.readonly and not os.path.exists(self.location):
 			self._ensure_dirs()
 
-	def __getitem__(self, cpv):
-		fp = os.path.join(self.location, cpv)
+	def _getitem(self, cpv):
+		# Don't use os.path.join, for better performance.
+		fp = self.location + _os.sep + cpv
 		try:
-			myf = open(fp, "r")
+			myf = codecs.open(_unicode_encode(fp,
+				encoding=_encodings['fs'], errors='strict'),
+				mode='r', encoding=_encodings['repo.content'],
+				errors='replace')
 			try:
-				d = self._parse_data(myf, cpv)
-				if "_mtime_" not in d:
-					"""Backward compatibility with old cache that uses mtime
-					mangling."""
-					d["_mtime_"] = long(os.fstat(myf.fileno()).st_mtime)
+				lines = myf.read().split("\n")
+				if not lines[-1]:
+					lines.pop()
+				d = self._parse_data(lines, cpv)
+				if '_mtime_' not in d:
+					# Backward compatibility with old cache
+					# that uses mtime mangling.
+					d['_mtime_'] = long(_os.fstat(myf.fileno()).st_mtime)
 				return d
 			finally:
 				myf.close()
-		except (IOError, OSError), e:
+		except (IOError, OSError) as e:
 			if e.errno != errno.ENOENT:
 				raise cache_errors.CacheCorruption(cpv, e)
-			raise KeyError(cpv)
+			raise KeyError(cpv, e)
 
 	def _parse_data(self, data, cpv):
 		try:
-			d = dict(map(lambda x:x.rstrip("\n").split("=", 1), data))
-		except ValueError, e:
+			return dict( x.split("=", 1) for x in data )
+		except ValueError as e:
 			# If a line is missing an "=", the split length is 1 instead of 2.
 			raise cache_errors.CacheCorruption(cpv, e)
-		if "_eclasses_" in d:
-			d["_eclasses_"] = reconstruct_eclasses(cpv, d["_eclasses_"])
-		else:
-			d["_eclasses_"] = {}
-		return d
-		
-		for x in self._known_keys:
-			if x not in d:
-				d[x] = ''
-
-
-		return d
-
 
 	def _setitem(self, cpv, values):
 #		import pdb;pdb.set_trace()
 		s = cpv.rfind("/")
 		fp = os.path.join(self.location,cpv[:s],".update.%i.%s" % (os.getpid(), cpv[s+1:]))
-		try:	myf=open(fp, "w")
-		except (IOError, OSError), e:
+		try:
+			myf = codecs.open(_unicode_encode(fp,
+				encoding=_encodings['fs'], errors='strict'),
+				mode='w', encoding=_encodings['repo.content'],
+				errors='backslashreplace')
+		except (IOError, OSError) as e:
 			if errno.ENOENT == e.errno:
 				try:
 					self._ensure_dirs(cpv)
-					myf=open(fp,"w")
-				except (OSError, IOError),e:
+					myf = codecs.open(_unicode_encode(fp,
+						encoding=_encodings['fs'], errors='strict'),
+						mode='w', encoding=_encodings['repo.content'],
+						errors='backslashreplace')
+				except (OSError, IOError) as e:
 					raise cache_errors.CacheCorruption(cpv, e)
 			else:
 				raise cache_errors.CacheCorruption(cpv, e)
@@ -89,27 +99,25 @@ class database(fs_template.FsBased):
 		#update written.  now we move it.
 
 		new_fp = os.path.join(self.location,cpv)
-		try:	os.rename(fp, new_fp)
-		except (OSError, IOError), e:
+		try:
+			os.rename(fp, new_fp)
+		except (OSError, IOError) as e:
 			os.remove(fp)
 			raise cache_errors.CacheCorruption(cpv, e)
 
-
 	def _delitem(self, cpv):
 #		import pdb;pdb.set_trace()
 		try:
 			os.remove(os.path.join(self.location,cpv))
-		except OSError, e:
+		except OSError as e:
 			if errno.ENOENT == e.errno:
 				raise KeyError(cpv)
 			else:
 				raise cache_errors.CacheCorruption(cpv, e)
 
-
 	def __contains__(self, cpv):
 		return os.path.exists(os.path.join(self.location, cpv))
 
-
 	def __iter__(self):
 		"""generator for walking the dir struct"""
 		dirs = [self.location]
@@ -117,7 +125,7 @@ class database(fs_template.FsBased):
 		while len(dirs):
 			try:
 				dir_list = os.listdir(dirs[0])
-			except OSError, e:
+			except OSError as e:
 				if e.errno != errno.ENOENT:
 					raise
 				del e
@@ -133,4 +141,3 @@ class database(fs_template.FsBased):
 					continue
 				yield p[len_base+1:]
 			dirs.pop(0)
-
