@@ -1,13 +1,24 @@
-# Copyright 1999-2006 Gentoo Foundation
+# Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: $
+# $Id: manifest.py 14509 2009-10-07 05:05:29Z zmedico $
 
-import errno, os
+import codecs
+import errno
 
-import portage.versions, portage.const
-from portage.checksum import *
-from portage.exception import *
-from portage.util import write_atomic
+import portage
+portage.proxy.lazyimport.lazyimport(globals(),
+	'portage.checksum:hashfunc_map,perform_multiple_checksums,verify_all',
+	'portage.util:write_atomic',
+)
+
+from portage import os
+from portage import _encodings
+from portage import _unicode_decode
+from portage import _unicode_encode
+from portage.exception import DigestException, FileNotFound, \
+	InvalidDataType, MissingParameter, PermissionDenied, \
+	PortageException, PortagePackageException
+from portage.localization import _
 
 class FileNotInManifestException(PortageException):
 	pass
@@ -18,12 +29,16 @@ def manifest2AuxfileFilter(filename):
 	if "CVS" in mysplit:
 		return False
 	for x in mysplit:
-		if x.startswith("."):
+		if x[:1] == '.':
 			return False
-	return not filename.startswith("digest-")
+	return not filename[:7] == 'digest-'
 
 def manifest2MiscfileFilter(filename):
 	filename = filename.strip(os.sep)
+	if portage._glep_55_enabled:
+		pf, eapi = portage._split_ebuild_name_glep55(filename)
+		if pf is not None:
+			return False
 	return not (filename in ["CVS", ".svn", "files", "Manifest"] or filename.endswith(".ebuild"))
 
 def guessManifestFileType(filename):
@@ -53,23 +68,30 @@ def parseManifest2(mysplit):
 class ManifestEntry(object):
 	__slots__ = ("type", "name", "hashes")
 	def __init__(self, **kwargs):
-		for k, v in kwargs.iteritems():
+		for k, v in kwargs.items():
 			setattr(self, k, v)
-	def __cmp__(self, other):
-		if str(self) == str(other):
-			return 0
-		return 1
 
 class Manifest2Entry(ManifestEntry):
 	def __str__(self):
 		myline = " ".join([self.type, self.name, str(self.hashes["size"])])
-		myhashkeys = self.hashes.keys()
+		myhashkeys = list(self.hashes)
 		myhashkeys.remove("size")
 		myhashkeys.sort()
 		for h in myhashkeys:
 			myline += " " + h + " " + str(self.hashes[h])
 		return myline
 
+	def __eq__(self, other):
+		if not isinstance(other, Manifest2Entry) or \
+			self.type != other.type or \
+			self.name != other.name or \
+			self.hashes != other.hashes:
+			return False
+		return True
+
+	def __ne__(self, other):
+		return not self.__eq__(other)
+
 class Manifest(object):
 	parsers = (parseManifest2,)
 	def __init__(self, pkgdir, distdir, fetchlist_dict=None,
@@ -79,7 +101,7 @@ class Manifest(object):
 		    Do not parse Manifest file if from_scratch == True (only for internal use)
 			The fetchlist_dict parameter is required only for generation of
 			a Manifest (not needed for parsing and checking sums)."""
-		self.pkgdir = pkgdir.rstrip(os.sep) + os.sep
+		self.pkgdir = _unicode_decode(pkgdir).rstrip(os.sep) + os.sep
 		self.fhashdict = {}
 		self.hashes = set()
 		self.hashes.update(portage.const.MANIFEST2_HASH_FUNCTIONS)
@@ -121,13 +143,15 @@ class Manifest(object):
 		"""Parse a manifest.  If myhashdict is given then data will be added too it.
 		   Otherwise, a new dict will be created and returned."""
 		try:
-			fd = open(file_path, "r")
+			fd = codecs.open(_unicode_encode(file_path,
+				encoding=_encodings['fs'], errors='strict'), mode='r',
+				encoding=_encodings['repo.content'], errors='replace')
 			if myhashdict is None:
 				myhashdict = {}
 			self._parseDigests(fd, myhashdict=myhashdict, **kwargs)
 			fd.close()
 			return myhashdict
-		except (OSError, IOError), e:
+		except (OSError, IOError) as e:
 			if e.errno == errno.ENOENT:
 				raise FileNotFound(file_path)
 			else:
@@ -178,15 +202,15 @@ class Manifest(object):
 		return myhashdict
 
 	def _createManifestEntries(self):
-		mytypes = self.fhashdict.keys()
+		mytypes = list(self.fhashdict)
 		mytypes.sort()
 		for t in mytypes:
-			myfiles = self.fhashdict[t].keys()
+			myfiles = list(self.fhashdict[t])
 			myfiles.sort()
 			for f in myfiles:
 				myentry = Manifest2Entry(
 					type=t, name=f, hashes=self.fhashdict[t][f].copy())
-				myhashkeys = myentry.hashes.keys()
+				myhashkeys = list(myentry.hashes)
 				myhashkeys.sort()
 				for h in myhashkeys:
 					if h not in ["size"] + portage.const.MANIFEST2_HASH_FUNCTIONS:
@@ -197,7 +221,7 @@ class Manifest(object):
 		for t in self.fhashdict:
 			for f in self.fhashdict[t]:
 				if portage.const.MANIFEST2_REQUIRED_HASH not in self.fhashdict[t][f]:
-					raise MissingParameter("Missing %s checksum: %s %s" % (portage.const.MANIFEST2_REQUIRED_HASH, t, f))
+					raise MissingParameter(_("Missing %s checksum: %s %s") % (portage.const.MANIFEST2_REQUIRED_HASH, t, f))
 
 	def write(self, sign=False, force=False):
 		""" Write Manifest instance to disk, optionally signing it """
@@ -207,16 +231,19 @@ class Manifest(object):
 			update_manifest = True
 			if not force:
 				try:
-					f = open(self.getFullname(), "r")
+					f = codecs.open(_unicode_encode(self.getFullname(),
+						encoding=_encodings['fs'], errors='strict'),
+						mode='r', encoding=_encodings['repo.content'],
+						errors='replace')
 					oldentries = list(self._parseManifestLines(f))
 					f.close()
 					if len(oldentries) == len(myentries):
 						update_manifest = False
-						for i in xrange(len(oldentries)):
+						for i in range(len(oldentries)):
 							if oldentries[i] != myentries[i]:
 								update_manifest = True
 								break
-				except (IOError, OSError), e:
+				except (IOError, OSError) as e:
 					if e.errno == errno.ENOENT:
 						pass
 					else:
@@ -226,7 +253,7 @@ class Manifest(object):
 					"".join("%s\n" % str(myentry) for myentry in myentries))
 			if sign:
 				self.sign()
-		except (IOError, OSError), e:
+		except (IOError, OSError) as e:
 			if e.errno == errno.EACCES:
 				raise PermissionDenied(str(e))
 			raise
@@ -292,21 +319,35 @@ class Manifest(object):
 		cpvlist = []
 		pn = os.path.basename(self.pkgdir.rstrip(os.path.sep))
 		cat = self._pkgdir_category()
-		for pkgdir, pkgdir_dirs, pkgdir_files in os.walk(self.pkgdir):
+
+		pkgdir = self.pkgdir
+
+		for pkgdir, pkgdir_dirs, pkgdir_files in os.walk(pkgdir):
 			break
 		for f in pkgdir_files:
-			if f.endswith(".ebuild"):
-				mytype = "EBUILD"
+			try:
+				f = _unicode_decode(f,
+					encoding=_encodings['fs'], errors='strict')
+			except UnicodeDecodeError:
+				continue
+			if f[:1] == ".":
+				continue
+			pf = None
+			if portage._glep_55_enabled:
+				pf, eapi = portage._split_ebuild_name_glep55(f)
+			elif f[-7:] == '.ebuild':
 				pf = f[:-7]
-				ps = portage.versions.pkgsplit(pf)
+			if pf is not None:
+				mytype = "EBUILD"
+				ps = portage.versions._pkgsplit(pf)
 				cpv = "%s/%s" % (cat, pf)
 				if not ps:
 					raise PortagePackageException(
-						"Invalid package name: '%s'" % cpv)
+						_("Invalid package name: '%s'") % cpv)
 				if ps[0] != pn:
 					raise PortagePackageException(
-						"Package name does not " + \
-						"match directory name: '%s'" % cpv)
+						_("Package name does not "
+						"match directory name: '%s'") % cpv)
 				cpvlist.append(cpv)
 			elif manifest2MiscfileFilter(f):
 				mytype = "MISC"
@@ -314,9 +355,16 @@ class Manifest(object):
 				continue
 			self.fhashdict[mytype][f] = perform_multiple_checksums(self.pkgdir+f, self.hashes)
 		recursive_files = []
-		cut_len = len(os.path.join(self.pkgdir, "files") + os.sep)
-		for parentdir, dirs, files in os.walk(os.path.join(self.pkgdir, "files")):
+
+		pkgdir = self.pkgdir
+		cut_len = len(os.path.join(pkgdir, "files") + os.sep)
+		for parentdir, dirs, files in os.walk(os.path.join(pkgdir, "files")):
 			for f in files:
+				try:
+					f = _unicode_decode(f,
+						encoding=_encodings['fs'], errors='strict')
+				except UnicodeDecodeError:
+					continue
 				full_path = os.path.join(parentdir, f)
 				recursive_files.append(full_path[cut_len:])
 		for f in recursive_files:
@@ -387,10 +435,10 @@ class Manifest(object):
 			if not ok:
 				raise DigestException(tuple([self._getAbsname(ftype, fname)]+list(reason)))
 			return ok, reason
-		except FileNotFound, e:
+		except FileNotFound as e:
 			if not ignoreMissing:
 				raise
-			return False, "File Not Found: '%s'" % str(e)
+			return False, _("File Not Found: '%s'") % str(e)
 
 	def checkCpvHashes(self, cpv, checkDistfiles=True, onlyDistfiles=False, checkMiscfiles=False):
 		""" check the hashes for all files associated to the given cpv, include all
@@ -474,7 +522,9 @@ class Manifest(object):
 		mfname = self.getFullname()
 		if not os.path.exists(mfname):
 			return rVal
-		myfile = open(mfname, "r")
+		myfile = codecs.open(_unicode_encode(mfname,
+			encoding=_encodings['fs'], errors='strict'),
+			mode='r', encoding=_encodings['repo.content'], errors='replace')
 		lines = myfile.readlines()
 		myfile.close()
 		for l in lines:
