@@ -1,19 +1,25 @@
 # portage: news management code
 # Copyright 2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: news.py 11957 2008-11-15 22:17:01Z zmedico $
+# $Id: news.py 14327 2009-09-21 16:07:07Z arfrever $
 
 __all__ = ["NewsManager", "NewsItem", "DisplayRestriction",
 	"DisplayProfileRestriction", "DisplayKeywordRestriction",
 	"DisplayInstalledRestriction"]
 
+import codecs
 import logging
-import os
+import os as _os
 import re
+from portage import os
+from portage import _encodings
+from portage import _unicode_decode
+from portage import _unicode_encode
 from portage.util import apply_secpass_permissions, ensure_dirs, \
 	grabfile, normalize_path, write_atomic, writemsg_level
 from portage.data import portage_gid
 from portage.dep import isvalidatom
+from portage.localization import _
 from portage.locks import lockfile, unlockfile
 from portage.exception import InvalidLocation, OperationNotPermitted, \
 	PermissionDenied
@@ -46,9 +52,9 @@ class NewsManager(object):
 		#   execute) bits set. News files should be world readable.
 		self._uid = int(self.config["PORTAGE_INST_UID"])
 		self._gid = portage_gid
-		self._file_mode = 00064
-		self._dir_mode  = 00074
-		self._mode_mask = 00000
+		self._file_mode = 0o0064
+		self._dir_mode  = 0o0074
+		self._mode_mask = 0o0000
 
 		portdir = portdb.porttree_root
 		profiles_base = os.path.join(portdir, 'profiles') + os.path.sep
@@ -69,7 +75,7 @@ class NewsManager(object):
 	def _news_dir(self, repoid):
 		repo_path = self.portdb.getRepositoryPath(repoid)
 		if repo_path is None:
-			raise AssertionError("Invalid repoID: %s" % repoid)
+			raise AssertionError(_("Invalid repoID: %s") % repoid)
 		return os.path.join(repo_path, self.news_path)
 
 	def updateItems(self, repoid):
@@ -92,7 +98,8 @@ class NewsManager(object):
 
 		news_dir = self._news_dir(repoid)
 		try:
-			news = os.listdir(news_dir)
+			news = _os.listdir(_unicode_encode(news_dir,
+				encoding=_encodings['fs'], errors='strict'))
 		except OSError:
 			return
 
@@ -110,6 +117,17 @@ class NewsManager(object):
 
 			updates = []
 			for itemid in news:
+				try:
+					itemid = _unicode_decode(itemid,
+						encoding=_encodings['fs'], errors='strict')
+				except UnicodeDecodeError:
+					itemid = _unicode_decode(itemid,
+						encoding=_encodings['fs'], errors='replace')
+					writemsg_level(
+						"!!! Invalid encoding in news item name: '%s'\n" % \
+						itemid, level=logging.ERROR, noiselevel=-1)
+					continue
+
 				if itemid in skip:
 					continue
 				filename = os.path.join(news_dir, itemid,
@@ -148,16 +166,16 @@ class NewsManager(object):
 		If update is specified, updateNewsItems( repoid ) will be called to
 		check for new items.
 		"""
-		
+
 		if update:
 			self.updateItems(repoid)
-		
+
 		unread_filename = self._unread_filename(repoid)
 		unread_lock = None
 		try:
 			unread_lock = lockfile(unread_filename, wantnewlockfile=1)
 		except (InvalidLocation, OperationNotPermitted, PermissionDenied):
-			return 0
+			pass
 		try:
 			try:
 				return len(grabfile(unread_filename))
@@ -167,6 +185,7 @@ class NewsManager(object):
 			if unread_lock:
 				unlockfile(unread_lock)
 
+_formatRE = re.compile("News-Item-Format:\s*([^\s]*)\s*$")
 _installedRE = re.compile("Display-If-Installed:(.*)\n")
 _profileRE = re.compile("Display-If-Profile:(.*)\n")
 _keywordRE = re.compile("Display-If-Keyword:(.*)\n")
@@ -180,11 +199,10 @@ class NewsItem(object):
 	"display if arch: x86" and so forth.
 
 	Creation of a news item involves passing in the path to the particular news item.
-
 	"""
-	
+
 	def __init__(self, path, name):
-		""" 
+		"""
 		For a given news item we only want if it path is a file.
 		"""
 		self.path = path
@@ -199,24 +217,33 @@ class NewsItem(object):
 		and a vardb so we can look at installed packages).
 		Each restriction will pluck out the items that are required for it to match
 		or raise a ValueError exception if the required object is not present.
+
+		Restrictions of the form Display-X are OR'd with like-restrictions;
+		otherwise restrictions are AND'd.  any_match is the ORing and
+		all_match is the ANDing.
 		"""
 
 		if not self._parsed:
 			self.parse()
 
 		if not len(self.restrictions):
-			return True # no restrictions to match means everyone should see it
-		
+			return True
+
 		kwargs = \
 			{ 'vardb' : vardb,
 				'config' : config,
 				'profile' : profile }
 
-		for restriction in self.restrictions:
-			if restriction.checkRestriction(**kwargs):
-				return True
-			
-		return False # No restrictions were met; thus we aren't relevant :(
+		all_match = True
+		for values in self.restrictions.values():
+			any_match = False
+			for restriction in values:
+				if restriction.checkRestriction(**kwargs):
+					any_match = True
+			if not any_match:
+				all_match = False
+
+		return all_match
 
 	def isValid(self):
 		if not self._parsed:
@@ -224,30 +251,40 @@ class NewsItem(object):
 		return self._valid
 
 	def parse(self):
-		lines = open(self.path).readlines()
-		self.restrictions = []
+		lines = codecs.open(_unicode_encode(self.path,
+			encoding=_encodings['fs'], errors='strict'),
+			mode='r', encoding=_encodings['content'], errors='replace'
+			).readlines()
+		self.restrictions = {}
 		invalids = []
 		for i, line in enumerate(lines):
-			#Optimization to ignore regex matchines on lines that
-			#will never match
+			# Optimization to ignore regex matchines on lines that
+			# will never match
+			format_match = _formatRE.match(line)
+			if format_match is not None and format_match.group(1) != '1.0':
+				invalids.append((i + 1, line.rstrip('\n')))
+				break
 			if not line.startswith('D'):
 				continue
 			restricts = {  _installedRE : DisplayInstalledRestriction,
 					_profileRE : DisplayProfileRestriction,
 					_keywordRE : DisplayKeywordRestriction }
-			for regex, restriction in restricts.iteritems():
+			for regex, restriction in restricts.items():
 				match = regex.match(line)
 				if match:
-					self.restrictions.append(restriction(match.groups()[0].strip()))
-					if not self.restrictions[-1].isValid():
+					restrict = restriction(match.groups()[0].strip())
+					if not restrict.isValid():
 						invalids.append((i + 1, line.rstrip("\n")))
+					else:
+						self.restrictions.setdefault(
+							id(restriction), []).append(restrict)
 					continue
 		if invalids:
 			self._valid = False
 			msg = []
-			msg.append("Invalid news item: %s" % (self.path,))
+			msg.append(_("Invalid news item: %s") % (self.path,))
 			for lineno, line in invalids:
-				msg.append("  line %d: %s" % (lineno, line))
+				msg.append(_("  line %d: %s") % (lineno, line))
 			writemsg_level("".join("!!! %s\n" % x for x in msg),
 				level=logging.ERROR, noiselevel=-1)
 
@@ -266,7 +303,7 @@ class DisplayRestriction(object):
 		return True
 
 	def checkRestriction(self, **kwargs):
-		raise NotImplementedError('Derived class should over-ride this method')
+		raise NotImplementedError('Derived class should override this method')
 
 class DisplayProfileRestriction(DisplayRestriction):
 	"""
@@ -301,7 +338,7 @@ class DisplayInstalledRestriction(DisplayRestriction):
 	An Installation restriction where a particular item shall only be displayed
 	if the user has that item installed.
 	"""
-	
+
 	def __init__(self, atom):
 		self.atom = atom
 
